##################################################################################################
######################################## CLASS DEFINITION ######################################## 
##################################################################################################
Queue <- setRefClass(
  Class = "Queue", 
  fields = list(
    elements = "list"
  )
)

##################################################################################################
###################################### SETTERS AND GETTERS #######################################
##################################################################################################
Queue$accessors(c("elements"))

##################################################################################################
############################################# METHODS ############################################
##################################################################################################

##################################################################################################

# Initializes the Queue with an element.
#
# Args:
#   elem: The first element in the Queue (non mandatory argument)
#
# Returns:
#   The initialized Queue object.

Queue$methods (
  initialize=function(elem=NULL){
    if(!is.null(elem)) {
      elements[[1]] <<- elem
    }
  }  
)

##################################################################################################

# Inserts a new element at the end of the Queue.
#
# Args:
#   newElem: The element to be inserted.
#
# Returns:
#   The modified queue with the new element.

Queue$methods (
  insert=function(newElem){
    elements[[length(elements)+1]] <<- newElem
  }  
)

##################################################################################################

# Extracts an element from the head of the queue.
#
# Args:
#
# Returns:
#   The extracted element. The queue is also modified removing that element.

Queue$methods (
  get=function(){
    n = length(elements)
    elem <- elements[[1]]
    elements <<- elements[-1]
    return(elem)
  }  
)

##################################################################################################

# Shows relevant information about the Queue: name of its elements.
#
# Args:
#
# Returns: 
#   No arguments are returned. Only the information is displayed on the screen. 

Queue$methods (
  show=function(){
    cat("\n")
    for(elem in elements)   
      cat(elem$getName(), " ")
  }
)

##################################################################################################

# Checks if a queue is empty.
#
# Args:
#
# Returns: 
#   A boolean indicating if the queue is empty.

Queue$methods (
  empty=function(){
    length(elements)==0
  }  
)

##################################################################################################
